@@ -69,7 +69,6 @@ typedef struct {
 
   char*		buf;		/* NULL if unbuffered */
   int		bufsz, bufcnt, bufptr;
-  int 		last_err;
 } nessus_connection;
 
 /* 
@@ -153,23 +152,6 @@ static int data_left(soc)
 } /* data_left */
 #endif
 
-int
-stream_get_err(fd)
- int fd;
-{
- nessus_connection *p;
- 
- if(!NESSUS_STREAM(fd))
-    {
-     errno = EINVAL;
-     return -1;
-    }
-    
-    
- p = &(connections[fd - NESSUS_FD_OFF]);
- return p->last_err;
-}
-
 /*
  * Returns a free file descriptor
  */
@@ -193,8 +175,6 @@ get_connection_fd()
  return -1;
 }
 
-
-
 static int
 release_connection_fd(fd)
  int fd;
@@ -270,7 +250,6 @@ nessus_register_connection(s, ssl)
   p->port = 0;			/* just used for debug */
   p->fd = s;
   p->transport = (ssl != NULL) ? NESSUS_ENCAPS_SSLv23 : NESSUS_ENCAPS_IP;
-  p->last_err  = 0;
   return fd;
 }
 
@@ -628,10 +607,7 @@ open_SSL_connection(fp, timeout, cert, key, passwd, cert_names)
 	{
 	  d = tictac + timeout - time(NULL);
 	  if (d <= 0)
-	    {
-	    fp->last_err = ETIMEDOUT;
 	    return -1;
-            }
 	  to.tv_sec = d;
 	  to.tv_usec = 0;
 	  errno = 0;
@@ -644,10 +620,7 @@ open_SSL_connection(fp, timeout, cert, key, passwd, cert_names)
 	}
       while (ret < 0 && errno == EINTR);
       if (ret <= 0)
-	{
-	fp->last_err = ETIMEDOUT;
 	return -1;
-        }
     }
   /*NOTREACHED*/
 }
@@ -746,7 +719,6 @@ open_stream_connection(args, port, transport, timeout)
  fp->transport = transport;
  fp->timeout   = timeout;
  fp->port      = port;
- fp->last_err  = 0;
  set_ids_evasion_mode(args, fp);
 
  if(fp->options & NESSUS_CNX_IDS_EVASION_FAKE_RST)
@@ -801,43 +773,34 @@ failed:
 
 
 ExtFunc int
-open_stream_connection_unknown_encaps5(args, port, timeout, p, delta_t)
+open_stream_connection_unknown_encaps(args, port, timeout, p)
  struct arglist * args;
  unsigned int  port;
  int timeout, * p;
- int	*delta_t;		/* time, in micro-seconds */
 {
  int fd;
  int i;
-  struct timeval	tv1, tv2;
- static int encaps[] = {
 #ifdef HAVE_SSL
+ static int encaps[] = {
    NESSUS_ENCAPS_SSLv2,
    NESSUS_ENCAPS_TLSv1,
    NESSUS_ENCAPS_SSLv3,
+    };
 #endif
-    NESSUS_ENCAPS_IP
-  };
  
 #if DEBUG_SSL > 2
  fprintf(stderr, "[%d] open_stream_connection_unknown_encaps: TCP:%d; %d\n",
 	 getpid(), port,timeout);
 #endif
 
+#ifdef HAVE_SSL
  for (i = 0; i < sizeof(encaps) / sizeof(*encaps); i ++)
-    {
-      if (delta_t != NULL) (void) gettimeofday(&tv1, NULL);
    if ((fd = open_stream_connection(args, port, encaps[i], timeout)) >= 0)
      {
        *p = encaps[i];
 #if DEBUG_SSL > 2
        fprintf(stderr, "[%d] open_stream_connection_unknown_encaps: TCP:%d -> transport=%d\n", getpid(), port, *p);
 #endif
-	  if (delta_t != NULL)
-	    {
-	      (void) gettimeofday(&tv2, NULL);
-	      *delta_t = (tv2.tv_sec - tv1.tv_sec) * 1000000 + (tv2.tv_usec - tv1.tv_usec);
-	    }
        return fd;
      }
    else if (__port_closed)
@@ -847,17 +810,23 @@ open_stream_connection_unknown_encaps5(args, port, timeout, p, delta_t)
 #endif
        return -1;
      }
-    }
+
+ /* 
+  * No need to try to connect to the remote port twice
+  */
+ if(__port_closed)
   return -1;
+#endif 
+ if ((fd = open_stream_connection(args, port, NESSUS_ENCAPS_IP, timeout)) >= 0)
+ {
+  *p = NESSUS_ENCAPS_IP;
+#if DEBUG_SSL > 2
+       fprintf(stderr, "[%d] open_stream_connection_unknown_encaps: TCP:%d -> transport=%d\n", getpid(), port, *p);
+#endif
+  return fd;
  }
  
-ExtFunc int
-open_stream_connection_unknown_encaps(args, port, timeout, p)
- struct arglist * args;
- unsigned int  port;
- int timeout, * p;
-{
-  return open_stream_connection_unknown_encaps5(args, port, timeout, p, NULL);
+ return -1;
 }
 
 
@@ -967,7 +936,6 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
       fp = &(connections[fd - NESSUS_FD_OFF]);
       trp = fp->transport;
       realfd = fp->fd;
-      fp->last_err = 0;
       if (fp->timeout != -2)
 	timeout = fp->timeout;
     }
@@ -1018,17 +986,11 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      ret = recv(realfd, buf + total, max_len - total, waitall);
 	      if (ret < 0)
 		if (errno != EINTR)
-                  {
-		  fp->last_err = errno;
 		  return total;
-                  }
 		else
 		  ret = 0;
 	      else if (ret == 0) /* EOF */
-                {
-                fp->last_err = EPIPE;
 		return total;
-                }
 	      /*ret > 0*/
 	      total += ret; 
 	      if (min_len > 0 && total >= min_len)
@@ -1036,7 +998,6 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      flag = 0;
 	    }
 	}
-      if ( t >= timeout ) fp->last_err = ETIMEDOUT;
       return total;
     }
 
@@ -1061,22 +1022,24 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
       FD_SET(realfd, &fdr); FD_SET(realfd, &fdw); 
       now = then = time(NULL);
       for (t = 0; timeout <= 0 || t < timeout; t = now - then )
-	{	
+	{
           now = time(NULL);
 	  tv.tv_sec = INCR_TIMEOUT; tv.tv_usec = 0;
 	  select_status = select ( realfd + 1, &fdr, &fdw, NULL, &tv );
+#if 1 /* Possible workaround for an OpenSSL bug */
           if ( select_status == 0 )
           {
-      	   FD_ZERO(&fdr); FD_ZERO(&fdw);
+      	  FD_ZERO(&fdr); FD_ZERO(&fdw);
       	   FD_SET(realfd, &fdr); FD_SET(realfd, &fdw); 
           }
 	  else
+#endif
 	  if ( select_status > 0 )
 	    {
 	  ret = SSL_read(fp->ssl, buf + total, max_len - total);
 	  if (ret > 0)
 		{
-	          total += ret;
+	    total += ret;
 		  FD_SET(realfd, &fdr);
 		  FD_SET(realfd, &fdw); 
 		}
@@ -1086,8 +1049,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      if (ret <= 0)
 		{
 		  err = SSL_get_error(fp->ssl, ret);
-		  FD_ZERO(&fdr); 
-		  FD_ZERO(&fdw);
+		  FD_ZERO(&fdr); FD_ZERO(&fdw);
 		  switch (err)
 	   {
 		    case SSL_ERROR_WANT_READ:
@@ -1100,7 +1062,6 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 #if DEBUG_SSL > 2
 		      fprintf(stderr, "SSL_Connect[%d]: SSL_ERROR_WANT_WRITE\n", getpid());
 #endif
-		      FD_SET(realfd, &fdr);
 		      FD_SET(realfd, &fdw);
 		      break;
 
@@ -1108,14 +1069,12 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 #if DEBUG_SSL > 2
 		      fprintf(stderr, "SSL_Connect[%d]: SSL_ERROR_ZERO_RETURN\n", getpid());
 #endif
-		      fp->last_err = EPIPE;
 		      return total;
 
 		    default:
 #if DEBUG_SSL > 0
 		      sslerror2("SSL_read", err);
 #endif
-		      fp->last_err = EPIPE;
 		      return total;
 		    }
 		}
@@ -1132,7 +1091,6 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	  else if (total >= min_len)
 		return total;
 	}
-      if ( t >= timeout ) fp->last_err = ETIMEDOUT;
       return total;
 #endif
     default :
@@ -1244,7 +1202,6 @@ write_stream_connection4(fd, buf0, n, i_opt)
     }
 
  fp = &(connections[fd - NESSUS_FD_OFF]);
- fp->last_err = 0;
  
 #if DEBUG_SSL > 8
  fprintf(stderr, "> write_stream_connection(%d, 0x%x, %d, 0x%x) \tE=%d 0=0x%x\n",
@@ -1269,11 +1226,7 @@ write_stream_connection4(fd, buf0, n, i_opt)
        ret = send(fp->fd, buf + count, n - count, i_opt);
 
     if(ret <= 0)
-      {
-       if ( ret < 0 ) fp->last_err = errno;
-       else fp->last_err = EPIPE;
-       break;
-      }
+      break;
      
      count += ret;
     }
@@ -1316,7 +1269,6 @@ write_stream_connection4(fd, buf0, n, i_opt)
 #if DEBUG_SSL > 0
 		  sslerror2("SSL_write", err);
 #endif      
-		  fp->last_err = EPIPE;
   	break;
      }
 	      if (fp->timeout >= 0)
@@ -1335,7 +1287,6 @@ write_stream_connection4(fd, buf0, n, i_opt)
 #if DEBUG_SSL > 0
 		  nessus_perror("select");
 #endif
-		  fp->last_err = ETIMEDOUT;
 		  break;
 		}
 	    }
@@ -1545,8 +1496,7 @@ open_socket(struct sockaddr_in *paddr,
   fd_set		fd_w;
   struct timeval	to;
   int			soc, x;
-  int			opt;
-  unsigned int opt_sz;
+  int			opt, opt_sz;
 
   __port_closed = 0;
 
@@ -2224,6 +2174,8 @@ int internal_send(int soc, char * data, int msg_type )
  int len;
  int e;
  int ack;
+ fd_set rd;
+ struct timeval tv;
  
  if ( data == NULL )
 	data = "";
@@ -2257,6 +2209,8 @@ int internal_recv(int soc, char ** data, int * data_sz, int * msg_type )
  int e;
  char * buf = *data;
  int    sz  = *data_sz;
+ fd_set rd;
+ struct timeval tv;
  int type;
  int ack;
  
@@ -2278,7 +2232,7 @@ int internal_recv(int soc, char ** data, int * data_sz, int * msg_type )
  if ( len >= sz )
  {
   sz = len + 1;
-  buf = erealloc( buf, sz );
+  buf = erealloc( buf, len );
  }
 
  if ( len > 0 )
@@ -2307,23 +2261,3 @@ error:
  *data_sz = 0;
  return -1;
 }
-
-
-ExtFunc int stream_pending(int fd)
-{
-  nessus_connection * fp;
- if ( ! NESSUS_STREAM(fd) )
- {
-  errno = EINVAL;
-  return -1;
- }
- fp = &(connections[fd - NESSUS_FD_OFF]);
-
- if ( fp->bufcnt )
-        return fp->bufcnt;
-#ifdef HAVE_SSL
- else if ( fp->transport != NESSUS_ENCAPS_IP )
-        return SSL_pending(fp->ssl);
-#endif
- return 0;
-}
