@@ -28,11 +28,11 @@
  */
 ExtFunc
 int 
-comm_send_status(globals, hostname, action,curr,max)
+comm_send_status(globals, hostname, action, current,max)
   struct arglist * globals;
   char * hostname;
   char * action;
-  int curr, max;
+  int current, max;
 {
  struct arglist * prefs = arg_get_value(globals,"preferences");
  char * pref = arg_get_value(prefs, "ntp_short_status");
@@ -40,6 +40,9 @@ comm_send_status(globals, hostname, action,curr,max)
  ntp_caps* caps = arg_get_value(globals, "ntp_caps");
  int soc = (int)arg_get_value(globals, "global_socket");
  char buffer[2048];
+ int e, n = 0, l, ack;
+ fd_set rd, wr;
+ struct timeval tv;
  
  
  if (soc < 0 || soc > 1024) 
@@ -58,16 +61,16 @@ comm_send_status(globals, hostname, action,curr,max)
     {
     if(short_status)
       {
-      snprintf(buffer, sizeof(buffer), "s:%c:%s:%d:%d\n", action[0], hostname, curr, max);
+      snprintf(buffer, sizeof(buffer), "s:%c:%s:%d:%d\n", action[0], hostname, current, max);
       }
     else
      snprintf(buffer, sizeof(buffer),
 		"SERVER <|> STATUS <|> %s <|> %s <|> %d/%d <|> SERVER\n",
-		hostname, action, curr, max);
+		hostname, action, current, max);
     }
   else
    snprintf(buffer, sizeof(buffer), "SERVER <|> STAT <|> %s <|> %d/%d <|> SERVER\n",
-	      hostname, curr, max);
+	      hostname, current, max);
 	      
 	      
  internal_send(soc, buffer, INTERNAL_COMM_MSG_TYPE_DATA);
